//-------------------------------------------------------
//
//FREQUENZIMETRO CON PRESCALER
//
//DOPPIO PRESCALER
//MAX: x16
//
//FREQ MAX: 800 MHz
//
//by Marco Plessi (Tecnovolt - http://tecnovolt.altervista.org/ )
//licenza CC BY-NC-ND 3.0 IT
//
//libreria "FreqPeriod" by http://interface.khm.de
//-------------------------------------------------------

#include <LiquidCrystal.h>
#include <EEPROM.h>

#include "FreqPeriod.h"

double lfrq;    // frequenza ingresso arduino
long int pp;    //periodo ingressi
int ctrlcicli = 10;    //controllo cicli programma per printing del valore
int valctrl = 10;    //controllo del printing
double freqreal = 0;    //frequenza reale ingresso frequenzimetro (con prescaler e fattore di scala)
double freqprint = 0;    //valore di freq da stampare a display
int prescaleratt = 1;    //valore attuale del prescaler (0;4;16)
double fscala = 1;    //fattore di scala (1=Hz ; 1000=kHz ; 1000000=MHz)
int btctrl[4] = {0, 0, 0, 0};    //controllo pulsanti
int btpress[4] = {0, 0, 0, 0};    //pulsanti premuti
int i = 0;    //variabile per ciclo for
int menuact = 0;    //controllo menu su lcd

int ctrm21 = 1;    //var controllo menu 2.1

int posq = 0;    //posizione del cursore sul display per il printing della frequenza
double rpmreal = 0;    // valore rpm in modalità contagiri
int tipomisura = 1;    //tipo della misura da effettuare (1:freq ; 2:RPM)
int unitafreq = 0;    //unità di misura per la frequenza
int unitarpm = 1;    //unità di misura per i giri
int serialctrl = 0;    //controllo della comunicazione seriale (0 spenta ; 1 attiva)


int EEtipomisura = 1;
int EEunitafreq = 0;
int EEprescaler = 1;
int EEserialctrl = 0;

LiquidCrystal lcd(13, 12, 11, 10, 9, 8);


void setup() {
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(A0, INPUT_PULLUP);
  pinMode(A1, INPUT_PULLUP);
  pinMode(A2, INPUT_PULLUP);
  pinMode(A3, INPUT_PULLUP);
  Serial.begin(115200);
  lcd.begin(16,2);
  FreqPeriod::begin();
  lcd.clear();
  lcd.print("Frequenzimetro");
  lcd.setCursor(0,1);
  lcd.print(" by tecnovolt");
  delay(1500);
  lcd.clear();
  lcd.print(" Mod.");
  lcd.setCursor(0,1);
  lcd.print("FQ-16S3-800");
  delay(1200);
  lcd.clear();
  lcd.print("caricamento dati");
  lcd.setCursor(0,1);
  lcd.print("da EEPROM...");
  //caricamento dati dalla EEPROM interna
  EEtipomisura=EEPROM.read(0);  //tipo misura
  delay(5);
  tipomisura=EEtipomisura;
  EEunitafreq=EEPROM.read(1);  //unità di misura
  delay(5);
  unitafreq=EEunitafreq;
  EEprescaler=EEPROM.read(2);
  delay(5);
  prescaleratt=EEprescaler;
  EEserialctrl=EEPROM.read(3);
  delay(5);
  serialctrl=EEserialctrl;
  if(serialctrl==1){
    Serial.print("Frequenzimetro FQ-16S3-800 by Tecnovolt");
  }
  switch(EEprescaler){
    case 1:
  prescaleratt=1;
  digitalWrite(2, LOW);
  digitalWrite(3, LOW);
  delay(50);
  digitalWrite(4, HIGH);
  break;
    case 4:
  prescaleratt=4;
  digitalWrite(2, LOW);
  digitalWrite(4, LOW);
  delay(50);
  digitalWrite(3, HIGH);
  break;
    case 16:    
  prescaleratt=16;
  digitalWrite(3, LOW);
  digitalWrite(4, LOW);
  delay(50);
  digitalWrite(2, HIGH);
  break;
  }
    

}

void loop() {
  
  //inizio controllo pulsanti ..
  
  if(digitalRead(A0)==0){
    btctrl[0]=1;
  }
  if(btctrl[0]==1){
    if(digitalRead(A0)==1){
      btctrl[0]=0;
      btpress[0]=1;
    }
  }
  
  if(digitalRead(A1)==0){
    btctrl[1]=1;
  }
  if(btctrl[1]==1){
    if(digitalRead(A1)==1){
      btctrl[1]=0;
      btpress[1]=1;
    }
  }
  
  if(digitalRead(A2)==0){
    btctrl[2]=1;
  }
  if(btctrl[2]==1){
    if(digitalRead(A2)==1){
      btctrl[2]=0;
      btpress[2]=1;
    }
  }
  
  if(digitalRead(A3)==0){
    btctrl[3]=1;
  }
  if(btctrl[3]==1){
    if(digitalRead(A3)==1){
      btctrl[3]=0;
      btpress[3]=1;
    }
  }
  
  //fine controllo pulsanti ^^
    
  //menu 0 - schermata iniziale con misura e valore del prescaler
  
  if(menuact==0){
    pp=FreqPeriod::getPeriod();
      if (pp ){
        lfrq= 16000400.0 / pp;
      }
    freqreal=lfrq*prescaleratt;
    if(tipomisura==1){
      if(freqreal<=999999){
        posq=0;
      }
      if(freqreal<=99999){
        posq=1;
      }
      if(freqreal<=9999){
        posq=2;
      }
      if(freqreal<=999){
        posq=3;
        if(unitafreq==1){
          posq=2;
        }
      }
      if(freqreal<=99){
        posq=4;
        if(unitafreq==1){
          posq=2;
        }
      }
      if(freqreal<=9){
        posq=5;
        if(unitafreq==1){
          posq=2;
        }
      }
      if(serialctrl==1){
        Serial.print("Frequenza: ");
        Serial.print(freqreal);
      }
      switch(unitafreq){
        case 0:
        lcd.clear();
        lcd.setCursor(posq,0);
        freqprint=freqreal;
        lcd.print(freqprint,3);
        lcd.print(" Hz");
        if(serialctrl==1){
          Serial.print(" Hz");
        }
        break;
        case 1:
        lcd.clear();
        lcd.setCursor(posq,0);
        freqprint=freqreal*0.001;
        lcd.print(freqprint,4);
        lcd.print(" kHz");
        if(serialctrl==1){
          Serial.print(" kHz");
        }
        break;
        case 2:
        lcd.clear();
        lcd.setCursor(2,0);
        freqprint=freqreal*0.000001;
        lcd.print(freqprint,4);
        lcd.print(" MHz");
        if(serialctrl==1){
          Serial.print(" MHz");
        }
        break;
      }
    }
    if(tipomisura==2){
      rpmreal=freqreal*60;
      if(rpmreal<=9999999){
        posq=0;
      }
      if(rpmreal<=999999){
        posq=1;
      }
      if(rpmreal<=99999){
        posq=2;
      }
      if(rpmreal<=9999){
        posq=3;
      }
      if(rpmreal<=999){
        posq=4;
      }
      if(rpmreal<=99){
        posq=5;
      }
      if(rpmreal<=9){
        posq=6;
      }
      lcd.clear();
      lcd.setCursor(posq,0);
      lcd.print(rpmreal,1);
      lcd.print(" rpm");
      if(serialctrl==1){
        Serial.print("Giri: ");
        Serial.print(rpmreal);
        Serial.print(" rpm");
      }
    }
    lcd.setCursor(0,1),
    lcd.print(" prescaler: x");
    lcd.print(prescaleratt);
    if(serialctrl==1){
      Serial.print("   prescaler: x");
      Serial.println(prescaleratt);
    }
    if(btpress[0]==1){
      prescaleratt=1;
      digitalWrite(2, LOW);
      digitalWrite(3, LOW);
      delay(50);
      digitalWrite(4, HIGH);
    }
    if(btpress[1]==1){
      prescaleratt=4;
      digitalWrite(2, LOW);
      digitalWrite(4, LOW);
      delay(50);
      digitalWrite(3, HIGH);
    }
    if(btpress[2]==1){
      prescaleratt=16;
      digitalWrite(3, LOW);
      digitalWrite(4, LOW);
      delay(50);
      digitalWrite(2, HIGH);
    }
    if(btpress[3]==1){
      menuact=1;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 1 - principale:impostazioni
  
  if(menuact==1){
    lcd.clear();
    lcd.print(" menu principale");
    lcd.setCursor(0,1);
    lcd.print("  impostazioni");
    if(btpress[1]==1){
      menuact=2;
    }
    if(btpress[2]==1){
      menuact=0;
    }
    if(btpress[3]==1){
      menuact=11;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2 - principale:imp. eeprom
  
  if(menuact==2){
    lcd.clear();
    lcd.print(" menu principale");
    lcd.setCursor(0,1);
    lcd.print("  imp. EEPROM");
    if(btpress[0]==1){
      menuact=1;
    }
    if(btpress[1]==1){
      menuact=3;
    }
    if(btpress[2]==1){
      menuact=0;
    }
    if(btpress[3]==1){
      menuact=21;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 3 - principale:informazioni
  
  if(menuact==3){
    lcd.clear();
    lcd.print(" menu principale");
    lcd.setCursor(0,1);
    lcd.print("     info");
    if(btpress[0]==1){
      menuact=2;
    }
    if(btpress[2]==1){
      menuact=0;
    }
    if(btpress[3]==1){
      menuact=31;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //INIZIO SOTTOMENU
  
  //menu 1.1 - impostazioni:tipo misura
  if(menuact==11){
    lcd.clear();
    lcd.print("imp> tipo misura");
    lcd.setCursor(0,1);
    switch(tipomisura){
      case 1:
      lcd.print("frequenzimetro");
      break;
      case 2:
      lcd.print(" contagiri");
      break;
    }
    if(btpress[1]==1){
      menuact=12;
    }
    if(btpress[2]==1){
      menuact=1;
    }
    if(btpress[3]==1){
      tipomisura++;
      if(tipomisura>=3){
        tipomisura=1;
      }
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 1.2 - impostazioni:unità misura
  if(menuact==12){
    lcd.clear();
    lcd.print("imp> unita' freq");
    lcd.setCursor(0,1);
    lcd.print(" Hz   kHz   MHz ");
    switch(unitafreq){
      case 0:
      lcd.setCursor(0,1);
      lcd.print("[");
      lcd.setCursor(3,1);
      lcd.print("]");
      break;
      case 1:
      lcd.setCursor(5,1);
      lcd.print("[");
      lcd.setCursor(9,1);
      lcd.print("]");
      break;
      case 2:
      lcd.setCursor(11,1);
      lcd.print("[");
      lcd.setCursor(15,1);
      lcd.print("]");
      break;
    }
    if(btpress[0]==1){
      menuact=11;
    }
    if(btpress[1]==1){
      menuact=13;
    }
    if(btpress[2]==1){
      menuact=1;
    }
    if(btpress[3]==1){
      unitafreq++;
      if(unitafreq>=3){
        unitafreq=0;
      }
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 1.3 - comunicazione seriale ON OFF
  if(menuact==13){
    lcd.clear();
    lcd.print("imp> com seriale");
    lcd.setCursor(0,1);
    lcd.print("(115200 bps) ");
    switch(serialctrl){
      case 0:
      lcd.print("OFF");
      break;
      case 1:
      lcd.print(" ON");
      break;
    }
    if(btpress[0]==1){
      menuact=12;
    }
    if(btpress[2]==1){
      menuact=1;
    }
    if(btpress[3]==1){
      if(serialctrl==1){
        serialctrl=0;
      }else{
        serialctrl=1;
      }
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1 - impostazioni eeprom
  if(menuact==21){
    if(ctrm21==1){
      lcd.clear();
      lcd.print(" !!ATTENZIONE!!");
      delay(1000);
      lcd.clear();
      delay(50);
      lcd.print("  scrittura su");
      lcd.setCursor(0,1);
      lcd.print(" EEPROM interna!");
      delay(1800);
      lcd.clear();
      delay(50);
      lcd.print("  continuare?");
      ctrm21=0;
    }
    if(btpress[2]==1){
      menuact=2;
      ctrm21=1;
    }
    if(btpress[3]==1){
      menuact=211;
      ctrm21=1;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 211 - EE:tipo misura
  if(menuact==211){
    lcd.clear();
    lcd.print(" EEPROM interna");
    lcd.setCursor(0,1);
    lcd.print(" tipo di misura");
    if(btpress[1]==1){
      menuact=212;
    }
    if(btpress[2]==1){
      menuact=2;
    }
    if(btpress[3]==1){
      menuact=2111;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.1.1 - tipo misura (EEi)
  if(menuact==2111){
    lcd.clear();
    lcd.print("tipo di misura");
    lcd.setCursor(0,1);
    switch(EEtipomisura){
      case 1:
      lcd.print("frequenzimetro");
      break;
      case 2:
      lcd.print("contagiri");
      break;
    }
    if(btpress[0]==1){
      EEtipomisura=1;
    }
    if(btpress[1]==1){
      EEtipomisura=2;
    }
    if(btpress[2]==1){
      menuact=211;
    }
    if(btpress[3]==1){
      EEPROM.write(0, EEtipomisura);
      delay(10);
      tipomisura=EEtipomisura;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.2 EE:unitafreq
  if(menuact==212){
    lcd.clear();
    lcd.print(" EEPROM interna");
    lcd.setCursor(0,1);
    lcd.print("unita' di misura");
    if(btpress[0]==1){
      menuact=211;
    }
    if(btpress[1]==1){
      menuact=213;
    }
    if(btpress[2]==1){
      menuact=2;
    }
    if(btpress[3]==1){
      menuact=2121;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.2.1 - unita di misura (EEi)
  if(menuact==2121){
    lcd.clear();
    lcd.print("unita' di misura");
    lcd.setCursor(0,1);
    lcd.print(" Hz   kHz   MHz");
    switch(EEunitafreq){
      case 0:
      lcd.setCursor(0,1);
      lcd.print("[");
      lcd.setCursor(3,1);
      lcd.print("]");
      break;
      case 1:
      lcd.setCursor(5,1);
      lcd.print("[");
      lcd.setCursor(9,1);
      lcd.print("]");
      break;
      case 2:
      lcd.setCursor(11,1);
      lcd.print("[");
      lcd.setCursor(15,1);
      lcd.print("]");
      break;
    }
    if(btpress[0]==1){
      if(EEunitafreq>0){
        EEunitafreq--;
      }else{
        EEunitafreq=2;
      }
    }
    if(btpress[1]==1){
      if(EEunitafreq<2){
        EEunitafreq++;
      }else{
        EEunitafreq=0;
      }
    }
    if(btpress[2]==1){
      menuact=212;
    }
    if(btpress[3]==1){
      EEPROM.write(1, EEunitafreq);
      unitafreq=EEunitafreq;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.3 - EE:prescaler
  if(menuact==213){
    lcd.clear();
    lcd.print(" EEPROM interna");
    lcd.setCursor(0,1);
    lcd.print("   prescaler");
    if(btpress[0]==1){
      menuact=212;
    }
    if(btpress[1]==1){
      menuact=214;
    }
    if(btpress[2]==1){
      menuact=2;
    }
    if(btpress[3]==1){
      menuact=2131;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.3.1 - prescaler predefinito (EEi)
  if(menuact==2131){
    lcd.clear();
    lcd.print("  prescaler");
    lcd.setCursor(0,1);
    lcd.print(" x1   x4   x16");
    if(EEprescaler!=1){
      if(EEprescaler!=4){
        if(EEprescaler!=16){
          EEprescaler=1;
        }
      }
    }
    switch(EEprescaler){
      case 1:
      lcd.setCursor(0,1);
      lcd.print("[");
      lcd.setCursor(3,1);
      lcd.print("]");
      break;
      case 4:
      lcd.setCursor(5,1);
      lcd.print("[");
      lcd.setCursor(8,1);
      lcd.print("]");
      break;
      case 16:
      lcd.setCursor(10,1);
      lcd.print("[");
      lcd.setCursor(14,1);
      lcd.print("]");
      break;
    }
    if(btpress[1]==1){
      EEprescaler=EEprescaler*4;
      if(EEprescaler>16){
        EEprescaler=1;
      }
    }
    if(btpress[2]==1){
      menuact=213;
      EEprescaler=prescaleratt;
    }
    if(btpress[3]==1){
      EEPROM.write(2, EEprescaler);
      prescaleratt=EEprescaler;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 3.1 - informazioni
  
  if(menuact==31){
    lcd.clear();
    lcd.print("frequenzimetro");
    lcd.setCursor(0,1);
    lcd.print("by TECNOVOLT");
    delay(1500);
    lcd.clear();
    lcd.print(" Mod.");
    lcd.setCursor(0,1);
    lcd.print("FQ-16S3-800");
    delay(1200);
    lcd.clear();
    lcd.print("www.tecnovolt");
    lcd.setCursor(0,1);
    lcd.print(".altervista.org");
    delay(2000);
    menuact=0;
  }
      
  //menu 2.1.4 - EE:serial l15200
  
  if(menuact==214){
    lcd.clear();
    lcd.print(" EEPROM interna");
    lcd.setCursor(0,1);
    lcd.print(" serial 115200");
    if(btpress[0]==1){
      menuact=213;
    }
    if(btpress[2]==1){
      menuact=2;
    }
    if(btpress[3]==1){
      menuact=2141;
    }
    i=0;
    for(i=0;i<4;i++){
      btpress[i]=0;
    }
  }
  
  //menu 2.1.4.1 - seriale attiva/spenta (EEi)
  
  if(menuact==2141){
    lcd.clear();
    lcd.print(" serial 115200");
    lcd.setCursor(0,1);
    lcd.print("  OFF    ON");
    switch(EEserialctrl){
      case 0:
      lcd.setCursor(1,1);
      lcd.print("[");
      lcd.setCursor(5,1);
      lcd.print("]");
      break;
      case 1:
      lcd.setCursor(8,1);
      lcd.print("[");
      lcd.setCursor(11,1);
      lcd.print("]");
      break;
    }
    if(btpress[0]==1){
      EEserialctrl=0;
    }
    if(btpress[1]==1){
      EEserialctrl=1;
    }
    if(btpress[2]==1){
      EEserialctrl=serialctrl;
      menuact=214;
    }
    if(btpress[3]==1){
      EEPROM.write(3, EEserialctrl);
      serialctrl=EEserialctrl;
    }
  }
      
      
  
  delay(50);
}

//***************************************************************************
void printDouble( double val, byte precision){
  // prints val with number of decimal places determine by precision
  // precision is a number from 0 to 6 indicating the desired decimial places
  // example: lcdPrintDouble( 3.1415, 2); // prints 3.14 (two decimal places)

  if(val < 0.0){
    Serial.print('-');
    val = -val;
  }

  Serial.print (int(val));  //prints the int part
  if( precision > 0) {
    Serial.print("."); // print the decimal point
    unsigned long frac;
    unsigned long mult = 1;
    byte padding = precision -1;
    while(precision--)
      mult *=10;

    if(val >= 0)
      frac = (val - int(val)) * mult;
    else
      frac = (int(val)- val ) * mult;
    unsigned long frac1 = frac;
    while( frac1 /= 10 )
      padding--;
    while(  padding--)
      Serial.print("0");
    Serial.print(frac,DEC) ;
  }
}


